#!/bin/bash

PATH=/usr/local/sbin:/usr/local/bin:/sbin:/bin:/usr/sbin:/usr/bin

__SENSORS() {

	[ ! -f '/usr/bin/sensors' ] && exit

	case "$1" in
		cpu)
			case "$(awk -F ': ' '/model name/{print $2}' /proc/cpuinfo)" in
				*D2550*) sensors coretemp-isa-* lm86-i2c-0-* 2> /dev/null | awk '/Core [0-9]:/{print $3}' | tr -cd '\11\12\15\40-\176' | tr -d "+|C" | sort -n | tail -n 1 ;;
				*J1800*) sensors coretemp-isa-* lm86-i2c-0-* 2> /dev/null | awk '/Core [0-9]:/{print $3}' | tr -cd '\11\12\15\40-\176' | tr -d "+|C" | sort -n | tail -n 1 ;;
				*J1900*) sensors coretemp-isa-* lm86-i2c-0-* 2> /dev/null | awk '/Core [0-9]:/{print $3}' | tr -cd '\11\12\15\40-\176' | tr -d "+|C" | sort -n | tail -n 1 ;;
				*Geode*) sensors lm86-i2c-*-* 2> /dev/null | awk '/temp[0-9]:/{print $2}' | tr -d ",|°C|+" | sort -n | tail -n 1 ;;
			esac
		;;

		mb)
			t_mb="$(sensors w83627dhg-isa-* it8718-isa-* 2> /dev/null | grep -v "disabled" | awk '/temp[0-9]:/{print $2}' | tr -d "+|°C|")"
			[ "$t_mb" ] && echo "$t_mb" | awk 'BEGIN{count=0;sum=0}{count+=1;sum+=$1}END{print sum/count}'
		;;

		ssd)
			[ "$(sudo -l | grep -w smartctl)" ] && sudo smartctl -A "$(awk '/rw2/{print $1}' /proc/mounts | tr -d '0-9')" | awk '/Reallocated_Sector/{print $10}'
		;;
	esac

}


__KAMAILIO() {

	case "$1" in
		stats)
			local GET_MEMORY="$(kamcmd core.shmmem m 2> /dev/null)"
			[ ! "$GET_MEMORY" ] && exit
			local SYS_MEMORY="$(( $(awk '/MemTotal:/{print $2}' /proc/meminfo) / 1024 ))"
			local SHM_MEMORY_USED="$(awk '/real_used:/{print $2}' <<< "$GET_MEMORY")"
			local SHM_MEMORY_TOTAL="$(awk '/total:/{print $2}' <<< "$GET_MEMORY")"

			case "$2" in
				usage_mem_p) echo "$(( $SHM_MEMORY_USED * 100 / $SHM_MEMORY_TOTAL ))" ;;
				req_mem_p)   echo "$(( $SHM_MEMORY_TOTAL * 100 / $SYS_MEMORY ))" ;;
			esac
		;;

		users)
			case "$(/usr/sbin/kamailio -v | awk '/version:/{print $3}')" in
				4.*) /usr/sbin/kamctl online 2> /dev/null | grep -wEc "[0-9]{1,10}" ;;
				5.*) /usr/sbin/kamcmd ul.dump brief 2> /dev/null | awk '/Records/{print $2}' ;;
			esac
		;;
	esac

}


__CALLWEAVER() {

	case "$1" in
		check_trunk)
			/usr/sbin/timeout 3 /usr/sbin/callweaver -rx 'sip show registry' 2> /dev/null | awk '/Request|Registered|Auth./{print $4" "$6}' | grep -w "$2" | while read PEERNAME STATE; do
				[ "$STATE" == "Registered" ] && echo "1" || echo "0"
			done
		;;

		discovery)
			echo -n '{"data":['
				/usr/sbin/timeout 3 /usr/sbin/callweaver -rx 'sip show registry' 2> /dev/null | awk '/Registered|Request Sent/{print $1" "$4}' | sed 's/%20/ /g' | while read TRUNK_IP TRUNK; do

					[ "$(grep -w "ws2sip.wildix.com" <<< $TRUNK_IP)" ] && TRUNKNAME="kite-connection" || TRUNKNAME="$TRUNK"
					echo -n "{\"{#TRUNKNAME}\": \"$TRUNKNAME\",\"{#TRUNK}\": \"$TRUNK\"},"

				done | sed 's/\},$/\}/g'
			echo -n ']}'
		;;

		stats)
			local NEW
			local OLD
			local DIFF
			local NEW_STATS='/tmp/cw.stat'
			local OLD_STATS='/tmp/cw.stat.old'

			[ ! -s $NEW_STATS ] && exit

			NEW="$(awk '/processed/{print $1}' $NEW_STATS)"

			[ ! -s $OLD_STATS ] && echo "${NEW}:${NEW}" > $OLD_STATS

			OLD="$(awk -F ':' '{print $1}' $OLD_STATS)"

				if [ "$NEW" -lt "$OLD" ]; then
					DIFF="$NEW"
				elif [ "$NEW" -gt "$OLD" ]; then
					DIFF="$(($NEW - $OLD))"
				else
					DIFF="0"
				fi

			[ "$(($(date +%s) - $(stat -c %Y $OLD_STATS)))" -ge "600" ] && echo "${NEW}:${DIFF}" > $OLD_STATS && echo "$DIFF" || awk -F ':' '{print $2}' $OLD_STATS
			;;
	esac

}


__EJABBERD() {

	case "$1" in
		stats) [ "$(sudo -l | grep -w ejabberdctl)" ] && sudo -u ejabberd -H /usr/sbin/ejabberdctl connected_users_number 2> /dev/null | grep -wE "^[0-9]{1,10}" ;;
	esac

}


__CHECK_REBOOT() {

	local PID="$2"
	local PID_TMP="$1"

	[ ! -f "$PID" ] && exit
	[ ! -f "$PID_TMP" ] && cat "$PID" > "$PID_TMP"

	if [ "$(cat "$PID")" != "$(cat "$PID_TMP")" ]; then
		cat $PID > $PID_TMP
		echo "1"
	else
		echo "0"
	fi

}

__SYSTEM_USAGE() {

	local USAGE='0'

	if [ "$2" ]; then
		case "$1" in
			la|LA)   USAGE="$(top -b -n 1 | awk '$12 ~ "'"${2}$"'" { sum+=$9 } END {print sum}')" ;;
			ram|RAM) USAGE="$(ps -eo rss,comm | awk '$2 ~ "'"${2}$"'" { sum+=$1 } END {print sum / 1024}')" ;;		
		esac
	fi

	[ "$USAGE" ] && echo "$USAGE" || echo "0"

}

__ZBA_UPDATE() {

	## update zba_update script
	case "$(cat /etc/debian_version 2>/dev/null)" in
		5.*)
				if [ "$(md5sum /etc/zabbix/zba_update | awk '{print $1}')" != "c70de5746d9e5706f95775985b660851" ]; then
					wget --timeout=1 --tries=2 https://zba-update.wildix.com/zba_update_5 -O /etc/zabbix/zba_update -o /dev/null
						if [ "$?" -eq "0" ]; then
							sudo chown -R wms:wms /etc/zabbix/
							sudo chmod +x /etc/zabbix/zba_update
							sudo rm -rf /etc/zabbix/hash
						fi
				fi
		;;
		9.*)
				if [ "$(md5sum /etc/zabbix/zba_update | awk '{print $1}')" != "00429674780e4c3d297c61f24a4d9e56" ]; then
					wget --timeout=1 --tries=2 https://zba-update.wildix.com/zba_update_9 -O /etc/zabbix/zba_update -o /dev/null
						if [ "$?" -eq "0" ]; then
							sudo chown -R wms:wms /etc/zabbix/
							sudo chmod +x /etc/zabbix/zba_update
							sudo rm -rf /etc/zabbix/hash
						fi
				fi
		;;
	esac

}

__APPLY_UPDATE() {

	local CONF='/etc/zabbix/zabbix_agentd.conf'

	# clear old config
	[ -f "/etc/zabbix/zabbix_agentd.d/callweaver.sh" ] && rm -f "/etc/zabbix/zabbix_agentd.d/callweaver.sh"
	[ -f "/etc/zabbix/zabbix_agentd.d/kamailio.sh" ] && rm -f "/etc/zabbix/zabbix_agentd.d/kamailio.sh"
	[ -f "/etc/zabbix/zabbix_agentd.d/sensor.sh" ] && rm -f "/etc/zabbix/zabbix_agentd.d/sensor.sh"
	[ -f "/etc/zabbix/zba_update.php" ] && rm -f "/etc/zabbix/zba_update.php"

	# change access domain
	if [ ! "$(grep -oE "ServerActive=zabbix[0-9]-" /etc/zabbix/zabbix_agentd.conf)" ]; then
		sed -i "s/^ServerActive=.*/ServerActive=zabbix$(dpkg -l | awk '/wms-interface-sqlite3/{print $3}' | sed 's/\./-/g' | grep -oE '^[0-9]-[0-9]{2}').wildix.com:8099/g" $CONF
		bash -c "sleep 5 && sudo killall -9 zabbix_agentd &>/dev/null &"
	fi

	## fix zabbix
	if [ "$(ps aux | grep -c "[z]abbix_agentd -c")" -gt "1" ] || [ "$(ps aux | grep -c "[z]abbix_agentd: active")" -gt "3" ]; then
		for PID in $(ps aux | grep "[z]abbix_agentd" | awk '{print $2}'); do
			[ "$ALL_PID" ] && ALL_PID="$ALL_PID $PID" || ALL_PID="$PID"
		done
		bash -c "sleep 5 && sudo kill -9 $ALL_PID &>/dev/null &"
	fi

}


case "$1" in
	sensor) __SENSORS "$2" ;;
	kam)    __KAMAILIO "$2" "$3" ;;
	call)   __CALLWEAVER "$2" "$3" ;;
	xmpp)   __EJABBERD "$2" ;;
	reboot)
		__CHECK_REBOOT "$2" "$3"
		__ZBA_UPDATE
	 ;;
	load)   __SYSTEM_USAGE LA "$2" ;;
	ram)    __SYSTEM_USAGE RAM "$2" ;;
	apply_update) __APPLY_UPDATE ;;
esac
